{*      Simple worm game for demonstrating Pascal syntax and
 *      program structure.
 *      Author: Timo Petmanson
 *      26. april 2008
 *}
program Worm;
uses
        keyboard, crt;
const
        KEY_UP = 65313;
        KEY_DOWN = 65319;
        KEY_LEFT = 65315;
        KEY_RIGHT = 65317;
        KEY_ESC = 283;
        KEY_SPACE = 14624;
        KEY_NONE = 0;
        MAX_W = 80;
        MAX_H = 24;
type
        Buffer = array[1..MAX_H,1..MAX_W] of Char;
        Direction = (UP, DOWN, LEFT, RIGHT);
        Point = record
                x, y, color : integer;
        end;
var
        levelData : Buffer;
        wormChars : array[1..5] of Char;
        menuKey : TKeyEvent;
        score : integer;



procedure LoadLevelData();
var
        f : Text;
        c : Char;
        i, j : Integer;
        line : array[1..MAX_W] of Char;
begin
        assign(f, 'level.dat');
        reset(f);
        j := 1;
        while j <= MAX_H do
        begin
                readln(f, line);
                for i:= 1 to MAX_W do
                        levelData[j,i] := line[i];
                j := j + 1;
        end;
        close(f);
end;


procedure DrawLevel();
var
        i, j : integer;
begin
        gotoXY(1,1);
        TextColor(red);
        TextBackGround(white);
        for j:= 1 to MAX_H do
                writeln(levelData[j][1..MAX_W]);
end;

{* This function processes user input *}
procedure ProcessUserInput(VAR dir : Direction; VAR gameEnd : boolean);
var
        key : TKeyEvent;
        code : Word;
        origDir : Direction;
begin
        origDir := dir;
        while TRUE do
        begin
                key := PollKeyEvent;
                if key = 0 then break;
                key := GetKeyEvent;
                key := TransLateKeyEvent(key);
                code := GetKeyEventCode(key);

                case code of
                        KEY_ESC: gameEnd := TRUE;
                        KEY_UP: if origDir <> DOWN then dir := UP;
                        KEY_DOWN: if origDir <> UP then dir := DOWN;
                        KEY_LEFT: if origDir <> RIGHT then dir := LEFT;
                        KEY_RIGHT: if origDir <> LEFT then dir := RIGHT;
                end;
        end;
end;


function Play() : integer;
var
        x, y : integer;
        dir : Direction;
        gameEnd : boolean;
        wormData : array[1..MAX_W*MAX_H] of Point;
        wormLen, score : integer;

        procedure CreateWorm(startx, starty, length : integer);
        var
                i : integer;
        begin
                for i := 1 to length do
                begin
                        wormData[i].x := startx;
                        wormData[i].y := starty;
                end;
        end;

        procedure ShiftWormData(newx, newy : integer);
        var
                i : integer;
        begin
                for i := wormLen downto 2 do
                        wormData[i] := wormData[i-1];
                wormData[1].x := newx;
                wormData[1].y := newy;
        end;

        procedure GrowWorm(units : integer);
        var
                i : integer;
        begin
                for i := 1 to units do
                        wormData[wormLen+i] := wormData[wormLen];
                wormLen := wormLen + units;
        end;

        procedure DrawRandApple();
        var
                x, y : integer;
        begin
                x := random(77)+2;
                y := random(22)+2;
                gotoxy(x,y);
                if levelData[y,x] <> ' ' then DrawRandApple
                else
                        begin
                                TextColor(red);
                                TextBackground(green);
                                write('A');
                                levelData[y,x] := 'A';
                        end;

        end;

        procedure DrawScore();
        begin
                gotoxy(1, 25);
                TextColor(white);
                TextBackground(black);
                write('Score: ', score);
        end;
begin
        x := 10;
        y := 10;
        dir := RIGHT;
        gameEnd := FALSE;
        wormlen := 10;
        createWorm(x, y, wormLen);
        cursoroff;
        score := 0;
        DrawRandApple;
        DrawScore;
        while not gameEnd do
        begin
                ProcessUserInput(dir, gameEnd);
                // first erase the tail
                gotoxy(wormData[wormLen].x, wormData[wormLen].y);
                ShiftWormData(x, y);
                TextBackground(white);
                Write(' ');
                levelData[wormData[wormLen].y, wormData[wormLen].x] := ' ';

                // kas s6i m6ne 6una 2ra
                if levelData[y,x] = 'A' then
                begin
                        score := score + 1;
                        GrowWorm(5);
                        DrawRandApple;
                        DrawRandApple;
                        DrawScore;
                end;
                // kas s6itis seina
                if (levelData[y,x] <> 'A') and (levelData[y,x] <> ' ') then
                        gameEnd := TRUE;
                // draw the head
                gotoxy(wormData[1].x, wormData[1].y);
                TextColor(blue);
                Textbackground(black);
                Write(wormchars[random(4)+1]);
                levelData[wormData[1].y, wormData[1].x] := 'O';

                case dir of
                        UP: y := y-1;
                        DOWN: y := y+1;
                        LEFT: x := x-1;
                        RIGHT: x := x+1;
                end;
                delay(75);
        end;
        Play := score;

end;

procedure EmptyKeyBuffer();
begin
         while True do
             begin
                menuKey := PollKeyEvent;
                if menuKey = 0 then break;
                menuKey := GetKeyEvent;
             end;
end;


begin
        {* Init game *}

        ClrScr;
        InitKeyBoard;
        randomize;
        wormchars[1] := 'O';
        wormchars[2] := '0';
        wormchars[3] := '*';
        wormchars[4] := '+';
        wormchars[5] := '';

    while TRUE do
        begin
        // joonistame menyy
        ClrScr;
        TextColor(white);
        TextBackground(black);
        gotoxy(29, 5);
        Write('W O R M    G A M E');
        gotoxy(30, 8);
        Write(' [p] - Play game');
        gotoxy(30, 12);
        Write('[q] - Exit to DOS');

        menuKey := GetKeyEvent;
        menuKey := TranslateKeyEvent(menuKey);
        if GetKeyEventChar(menuKey) = 'p' then
        begin
             LoadLevelData;
             DrawLevel;
             score := Play;
             clrscr;
             TextColor(white);
             TextBackground(black);
             gotoxy(23, 8);
             writeln('Game over. You scored ', score, ' points!');
             EmptyKeyBuffer;
             menuKey := GetKeyEvent;
        end
        else if GetKeyEventChar(menuKey) = 'q' then break;
    end;
end.










